//--------------------------------------------------------------------------------
// New Tab Script
//--------------------------------------------------------------------------------
// This script contains logic for new tab page.
//
// TEN MINUTES LOCAL CACHE
var WALLPAPER_CACHE_TIME_MS = 1000*60*10;
var CURRENT_IMG_URL = '';

var UnsplashPhotosApi = (function() {
    function UnsplashPhotosApi() {}
    UnsplashPhotosApi.prototype.UnsplashPhotosApi = function() {};
    UnsplashPhotosApi.prototype.getWallpapers = function(callback) {
        var _this = this;
        chrome.storage.local.get(["wallpapers"], function(items) {
            if (items.wallpapers)
                return callback(items.wallpapers);
            return _this.fetchNewWallpapers(callback);
        });
    };
    UnsplashPhotosApi.prototype.fetchNewWallpapers = function(callback) {
        var params = {
            collections: "220388,402",
            orientation: "landscape",
            query: "nature",
            count: 30
        };
        $.get(UnsplashPhotosApi.BASE_URL, params, function(response) {
            chrome.storage.local.set({ wallpapers: response }, function() {
                callback(response);
            });
        });
    };
    UnsplashPhotosApi.getWallpaperUrl = function(photo) {
        if (photo.urls && photo.urls.full)
            return photo.urls.full;
        if (photo.urls && photo.urls.full)
            return photo.urls.full;
        if (photo.urls && photo.urls.raw)
            return photo.urls.raw;
        return "/data/background.jpg";
    };
    UnsplashPhotosApi.getUserProfileUrl = function(wallpaper) {
        return wallpaper.user.links.html + "?utm_source=brightTab&utm_medium=referral&utm_campaign=api-credit";
    };
    UnsplashPhotosApi.getUserFullName = function(wallpaper) {
        return wallpaper.user.first_name + " " + wallpaper.user.last_name;
    };
    return UnsplashPhotosApi;
}());
UnsplashPhotosApi.BASE_URL = "https://bright-tab.com/api/wallpapers_v2.php";


//----------------------------------------------------------------
// Weather functioins
//----------------------------------------------------------------
// Logic is that we cache weather for 5 hours. We do not need to
// fetch new data every time a user opens a new tab and weather will
// not change in a second (well there are situations but they are very
// rare, and this extension is not that proffessional that needs to
// take those use cases in considaration).


/**
 * Checks if cache entry is expired. It expects that provided object contains
 * `createdAt` property against check will be performed.
 *
 * @param Object cacheEntry
 */
function isCacheExpired(cacheEntry) {
    if (cacheEntry === null || cacheEntry === undefined)
        return true;

    if (cacheEntry.createdAt === null || cacheEntry.createdAt === undefined)
        return true;

    let createdAt = new Date(cacheEntry.createdAt);
    let now = new Date();
    let timeDiff = Math.abs(createdAt.getTime() - now.getTime());
    var diffHours = Math.ceil(timeDiff / (1000 * 3600));
    if (diffHours >= 5)
        return true;

    return false;

}

//----------------------------------------------------------------
// Main
//----------------------------------------------------------------

/**
 * Checks if wallpaper has expired. It exptects for entry to have
 * createdAt property.
 *
 * @param Object entry
 */
function isWallpaperExpired(entry) {
    if (entry === null || entry === undefined)
        return true;

    if (entry.createdAt === null || entry.createdAt === undefined)
        return true;

    let createdAt = new Date(entry.createdAt);
    let now = new Date();
    let timeDiff = Math.abs(createdAt.getTime() - now.getTime());
    var diffHours = Math.ceil(timeDiff / (1000 * 3600));
    if (diffHours >= 12)
        return true

    return false;
}

/**
 * Sets background and author.
 *
 * @param wallpaper Object
 */
function onWallpaper(wallpaper) {
    //$(document.body).css("background-image", "url(" + FlickrApi.getWallpaperUrl(wallpaper) + ")");
    CURRENT_IMG_URL =  UnsplashPhotosApi.getWallpaperUrl(wallpaper)
    $(document.body).css({
        "background-image" : "url(" + CURRENT_IMG_URL + ")",
        "height" : "100%",
        "background-repeat" : "no-repeat",
        "width" : "100%",
        "overflow-y": "hidden"
    });
    $("#uiBlur").css("background-image", "url(" + CURRENT_IMG_URL + ")")
    $("#PhotoByUrl")
        .attr("href", UnsplashPhotosApi.getUserProfileUrl(wallpaper))
        .text(UnsplashPhotosApi.getUserFullName(wallpaper));
    $("#PhotoBy").show();
    preloadNext()
}

function selectNewWallpaper(counter){
    var api = new UnsplashPhotosApi();
    api.getWallpapers(function (wallpapers) {
        if (!selectWallpaper(wallpapers))
            api.fetchNewWallpapers(selectWallpaper);
    });
}

function selectWallpaper(wallpapers) {
    for (let i = 0; i < wallpapers.length; i++) {
        let wallpaper = wallpapers[i];
        if (wallpaper.used !== true) {
            wallpaper.used = true;
            chrome.storage.local.set({
                wallpapers: wallpapers,
                currentWallpaper: wallpaper,
                currentWallpaperCreatedAt: Date.now()
            }, function () {
            });
            onWallpaper(wallpaper);
            return true;
        }
    }
    return false;
}

function preloadNext(){
    chrome.storage.local.get(["wallpapers"], function (items) {
        if (items.wallpapers){
            var preload_images = '';
            var wallpapers = items.wallpapers
            for (let i = 0; i < wallpapers.length; i++) {
                let wallpaper = wallpapers[i];
                if (wallpaper.used !== true) {
                    let image_url = UnsplashPhotosApi.getWallpaperUrl(wallpapers[i]);
                    preload_images += '<img style="display: none;" src="' + image_url + '"/>';
                    $(".preload-images").html(preload_images);
                    break
                }
            }
        }
    });
}

function startSearch() {
    let query = $("#SearchBar").val().trim();
    if (query.length === 0) {
        return;
    }
    window.location="http://bright-tab.com/search.php?q="+query
}

function todoShow() {
    chrome.storage.local.get(["userId", "todo", "incompleteTasks", "completedTasks", "todoOpen"], function (items) {
        var todo;
        if (items.userId) {
        }

        if (items.todo === true && items.todoOpen === "block") {

            document.getElementById("todo").style.display = "block";
            document.getElementById("content").style.display = "none";

            var incompleteTasks = items.incompleteTasks;
            var completedTasks = items.completedTasks;

            incompleteTasks = incompleteTasks.split("\n");
            incompleteTasks.filter(function (value) {
                if (value === "")
                    incompleteTasks.pop();
            });

            completedTasks = completedTasks.split("\n");
            completedTasks.filter(function (value) {
                if (value === "")
                    completedTasks.pop();
            });

            if (incompleteTasks !== "")
                for (var i = 0; i < incompleteTasks.length; i++) {
                    //Create a new list item with the text from the #new-task:
                    var listItem = createNewTaskElement(incompleteTasks[i]);
                    //Append listItem to incompleteTaskHolder
                    incompleteTasksHolder.appendChild(listItem);
                    bindTaskEvents(listItem, taskCompleted);
                }

            if (completedTasks !== "")
                for (var k = 0; k < completedTasks.length; k++) {
                    var listItem2 = createNewTaskElement(completedTasks[k]);
                    completedTasksHolder.appendChild(listItem2);
                    bindTaskEvents(listItem2, taskIncomplete);
                }


            $('#completed-tasks').find(':checkbox').each(function () {
                $(this).attr('checked', true);
            });

            todoCount = incompleteTasksHolder.children.length;
            todoCount = parseInt(todoCount);
            document.getElementById("todo-count").innerHTML = todoCount.toString() + " to do";
        }

    });

    var todo = document.getElementById("todo");
    todo.addEventListener("mouseleave", saveTodo);
}

$(document).ready(function() {

    todoShow();

    // Some visual effect when opening new tab
    $("html").fadeIn(600);

    var api = new UnsplashPhotosApi();
    api.getWallpapers(function (wallpapers) {
        if (!selectWallpaper(wallpapers))
            api.fetchNewWallpapers(selectWallpaper);
    });

    // event listener
    $(".bookmark-button").click(function (event) {
        let href = $(this).data("href");
        let name = $(this).data("name");


        chrome.tabs.create({url: href});
    });

    // event listener
    // $("#SearchBar").keypress(function (event) {
    //     if (event.keyCode === 13) {
    //         let query = $("#SearchBar").val().trim();
    //         if (query.length === 0)
    //             return;
    //         startSearch();
    //     }
    // });

    // event listener
    $(".show-more").click(function () {

        $("#details").slideToggle(function () {
            $(".show-more").toggleClass("active");
        });
    });

    var counter = 1;
    $("#logo-image").click(function(){
        selectNewWallpaper(counter);
        counter++;
        if(counter>4){
            counter = 1;
        }
    });
/*
    $('#exampleSlider').multislider({
        interval: 0,
        slideAll: false,
        duration: 1500
    });*/

    document.addEventListener("click", (evt) => {

        chrome.storage.local.get("todoOpen", function (items) {
            if (items.todoOpen === "block")
                todo_form_is_showed = true;
        });

        if(todo_form_is_showed){

            const flyoutElement = document.getElementById("todo");
            const todoImg = document.getElementById("todo-img");
            const todoButton = document.getElementById("todo-button");
            let targetElement = evt.target; // clicked element

            do {

                if (targetElement == flyoutElement || targetElement == todoImg ||
                    targetElement == todoButton || targetElement.className === "delete") {
                    // This is a click inside. Do nothing, just return.
                    //document.getElementById("flyout-debug").textContent = "Clicked inside!";
                    return;
                }
                // Go up the DOM
                targetElement = targetElement.parentNode;
            } while (targetElement);
            todoList.style.display = "none";
            todo_form_is_showed = false;
            chrome.storage.local.set({ todoOpen: "none" });
        }
    });

});

    //Clock

var $dOut = $('#date'),
    $hOut = $('#hours'),
    $mOut = $('#minutes'),
    $sOut = $('#seconds'),
    $ampmOut = $('#ampm');
var months = [
    'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'
];

var days = [
    'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'
];

function update(){
    var date = new Date();
    var ampm = date.getHours() < 12
        ? 'am'
        : 'pm';
    var hours = date.getHours();
    var minutes = date.getMinutes() < 10
        ? '0' + date.getMinutes()
        : date.getMinutes();

    var seconds = date.getSeconds() < 10
        ? '0' + date.getSeconds()
        : date.getSeconds();

    var dayOfWeek = days[date.getDay()];
    var month = months[date.getMonth()];

    var day = date.getDate();
    var year = date.getFullYear();

    var dateString =  dayOfWeek +", " + day + ' ' + month + " " + year;

    $dOut.text(dateString);
    $hOut.text(hours);
    $mOut.text(minutes);
    $sOut.text(seconds);
    $ampmOut.text(ampm);
}

update();
let timeout = 1000;
let calculate_second = true;

setTimeout(function run() {
    let current_date = new Date();
    let current_sec = current_date.getSeconds();

    if(calculate_second == true){
        timeout = (60-current_sec) * 1000;
        calculate_second = false;
    }
    else{
        timeout = 60000;
    }
    update();

    setTimeout(run, timeout); // else re-schedule
}, 0);

var isMac = navigator.platform.toUpperCase().indexOf('MAC') >= 0;

if (isMac)
    document.getElementById("arrow").style.right = "70px";

document.getElementById("darken_background").onclick = function(){
    document.getElementById("darken_background").style.display = "none";
    document.getElementById("mozilla").style.display = "none";
};

setTimeout(function() {
    document.getElementById("darken_background").style.display = "none";
    document.getElementById("mozilla").style.display = "none";
    }, 5000);

var todoList=document.getElementById("todo");
var taskInput = document.getElementById("new-task"); // new-task
var addButton = document.getElementById("AddButton");//first button
var saveButton = document.getElementById("SaveButton");//save button
var incompleteTasksHolder = document.getElementById("incomplete-tasks"); //incomplete-tasks
var completedTasksHolder = document.getElementById("completed-tasks"); //completed-tasks
var todoOpen;
var todoCount;
var todoCompletedCount;
var test_open=true;
var todo_form_is_showed=false;
//New Task List item
var j=0;

var createNewTaskElement = function(taskString) {
    // create List Item
    j++;
    var listItem = document.createElement("li");
    listItem.setAttribute("id","listItem"+j);
    listItem.addEventListener("mouseenter", showDeleteButton);
    listItem.addEventListener("mouseleave", hideDeleteButton);

    // input checkbox
    var checkBox = document.createElement("input");

    // label
    var label = document.createElement("label");
    label.setAttribute("id","label"+j);
    label.addEventListener('dblclick', editTask);

    // input (text)
    var editInput = document.createElement("input");
    editInput.setAttribute("id","input"+j);

    // button.edit
    var editButton = document.createElement("button");
    editButton.style.display = "none";

    // button.delete
    var deleteButton = document.createElement("button");
    var exitIcon = document.createElement("i");
    exitIcon.setAttribute("class", "fa fa-times");
    deleteButton.appendChild(exitIcon);
    deleteButton.style.display = "none";
    deleteButton.setAttribute("id","deleteButton"+j);
    //Each element needs modified
    checkBox.type = "checkBox";
    editInput.type = "text";
    editButton.innerText = "Edit";
    editButton.className = "edit";
    //deleteButton.innerHTML = "X";
    deleteButton.className = "delete";

    label.innerText = taskString;

    // Each element needs appending
    listItem.appendChild(checkBox);
    listItem.appendChild(label);
    listItem.appendChild(editInput);
    listItem.appendChild(editButton);
    listItem.appendChild(deleteButton);

    return listItem;
}


//Add a new task
var addTask = function() {
    if(taskInput.value===""){
        return
    }

    document.getElementById("content").style.display="none";

    //Create a new list item with the text from the #new-task:
    var listItem = createNewTaskElement(taskInput.value);
    //Append listItem to incompleteTaskHolder
    incompleteTasksHolder.appendChild(listItem);
    bindTaskEvents(listItem, taskCompleted);
    taskInput.value = "";
    todoCount=incompleteTasksHolder.children.length;
    todoCount = parseInt(todoCount);
    document.getElementById("todo-count").innerHTML= todoCount.toString() +" to do";
};

//Edit an existing task
var editTask = function() {
    var id = parseInt(this.id.match(/\d+/)[0]);
    var listItem = this.parentNode;
    var editInput = listItem.querySelector("input[type=text]");

    //editInput.focus();
    editInput.addEventListener('keypress',function (e) {
        var key = e.which || e.keyCode;
        if(key ===13){
            listItem.classList.value="";
            label.innerText=editInput.value;
        }
    });
    var label = listItem.querySelector("label");
    editInput.addEventListener('dblclick', editTask);
    var containsClass = listItem.classList.contains("editMode");

    // if class of the parent is .editMode
    if (containsClass) {
        //Switch from .editMode
        //label text become the input's value;
        label.innerText = editInput.value;
    } else {
        //Switch to .editMode
        //input value becomes the labels text
        editInput.value = label.innerText;
    }
    //Toggle .editMode on the parent
    listItem.classList.toggle("editMode");
    document.getElementById("input"+id).focus();
}

//Delete an existing task
var deleteTask = function () {
    //Remove the parent list item from the ul
    var listItem = this.parentNode;
    var ul = listItem.parentNode;
    i++;
    ul.removeChild(listItem);
    todoCount = incompleteTasksHolder.children.length;
    todoCompletedCount= completedTasksHolder.children.length;
    todoCount = parseInt(todoCount);
    document.getElementById("todo-count").innerHTML = todoCount.toString() +" to do";
    if(todoCount===0 && todoCompletedCount===0)
    {
        chrome.storage.local.set({todo: false, incompleteTasks: "", completedTasks: "" }, function() {
            });
            return

    }
    var incompleteTasks;
    var completedTasks;

    incompleteTasks = incompleteTasksHolder.innerText;
    completedTasks = completedTasksHolder.innerText;
    chrome.storage.local.set({todo: true, incompleteTasks: incompleteTasks, completedTasks: completedTasks }, function() {
    });
};

//Mark a task as complete
var taskCompleted = function() {
    //When the Checkbox is checked
    //Append the task list item to the #completed-tasks ul
    var listItem = this.parentNode;
    completedTasksHolder.appendChild(listItem);
    bindTaskEvents(listItem, taskIncomplete);
    todoCount=incompleteTasksHolder.children.length;
    todoCount = parseInt(todoCount);
    document.getElementById("todo-count").innerHTML = todoCount.toString() +" to do";
}

//Mark a task as incomplete
var taskIncomplete = function() {
    //When the checkbox is unchecked appendTo #incomplete-tasks
    var listItem = this.parentNode;
    incompleteTasksHolder.appendChild(listItem);
    bindTaskEvents(listItem, taskCompleted);
    todoCount=incompleteTasksHolder.children.length;
    todoCount = parseInt(todoCount);
    document.getElementById("todo-count").innerHTML = todoCount.toString() +" to do";
}


//Set the click handler to the addTask function
addButton.addEventListener("click", addTask);

var bindTaskEvents = function(taskListItem, checkBoxEventHandler) {
    // select listitems chidlren
    var checkBox = taskListItem.querySelector('input[type="checkbox"]');
    var editButton = taskListItem.querySelector("button.edit");
    var deleteButton = taskListItem.querySelector("button.delete");
    //bind editTask to edit button
    editButton.onclick = editTask;
    //bind deleteTask to delete button
    deleteButton.onclick = deleteTask;
    //bind checkBoxEventHandler to checkbox
    checkBox.onchange = checkBoxEventHandler;
}

document.addEventListener('DOMContentLoaded', function() {
    document.getElementById("todo-img").addEventListener("click", todoHandler);
    document.getElementById("todo-button").addEventListener("click", todoHandler);
});

function todoHandler() {
    var todoOpen;
    if(todoList.style.display === "none" || todoList.style.display === ""){
        todoList.style.display = "block";
        todo_form_is_showed = true;

    }
    else if(todoList.style.display === "block") {
        todoList.style.display = "none";
        todo_form_is_showed = false;
    }
    todoOpen = document.getElementById("todo").style.display;
    chrome.storage.local.set({ todoOpen:todoOpen });
    if(test_open){
    todoShow();
        test_open = false;
    }
}

document.addEventListener('DOMContentLoaded', function() {
    todoList=document.getElementById("todo");
    document.getElementById("todo-button").addEventListener("click", todoHandler);
});

document.addEventListener("keydown", function (e) {
    if (e.keyCode === 13) {
        addTask();
    }
});

function showDeleteButton () {
    var id = this.id.match(/\d+/);
    $("#deleteButton"+id).stop().fadeIn(1000);
}

function hideDeleteButton () {
    var id = this.id.match(/\d+/);

    $("#deleteButton"+id).stop().fadeOut(1000);
}

//cycle over incompleteTaskHolder ul list items
for (var i = 0; i < incompleteTasksHolder.children.length; i ++) {
    //bind events to list item's children (taskCompleted)
    bindTaskEvents(incompleteTasksHolder.children[i], taskCompleted);
}

//cycle over completedTaskHolder ul list items
for (var i = 0; i < completedTasksHolder.children.length; i ++) {
    //bind events to list item's children (taskCompleted)
    bindTaskEvents(completedTasksHolder.children[i], taskIncomplete);
}

function saveTodo() {
    todoCount = incompleteTasksHolder.children.length;
    todoCompletedCount = completedTasksHolder.children.length;

    if(todoCount === 0 && todoCompletedCount === 0)
    {
        return
    }

    var incompleteTasks;
    var completedTasks;
    var todoOpen;
    incompleteTasks = incompleteTasksHolder.innerText;
    completedTasks = completedTasksHolder.innerText;
    chrome.storage.local.set({todo: true, incompleteTasks: incompleteTasks, completedTasks: completedTasks }, function() {

    });
}

if (!document.getElementsByClassName) {
    document.getElementsByClassName = function (classname) {// Get a list of all elements in the document
        // For IE
        var node = document.body;
        var a = [];
        var re = new RegExp('(^| )'+classname+'( |$)');
        var els = node.getElementsByTagName("*");
        for(var i=0,j=els.length; i<j; i++)
            if(re.test(els[i].className))a.push(els[i]);
        return a;
    };
}

if (!Array.prototype.indexOf)
{
    Array.prototype.indexOf = function(elt /*, from*/)
    {
        var len = this.length >>> 0;

        var from = Number(arguments[1]) || 0;
        from = (from < 0)
            ? Math.ceil(from)
            : Math.floor(from);
        if (from < 0)
            from =  from + len;

        for (; from < len; from++)
        {
            if (from in this &&
                this[from] === elt)
                return from;
        }
        return -1;
    };
}

if (!('forEach' in Array.prototype)) {

    Array.prototype.forEach= function(action, that /*opt*/) {
        for (var i= 0, n= this.length; i<n; i++)
            if (i in this)
                action.call(that, this[i], i, this);
    };
}

if(typeof String.prototype.trim !== 'function') {
    String.prototype.trim = function() {
        return this.replace(/^\s+|\s+$/g, '');
    }
}

if (!Array.prototype.fill) {
    Array.prototype.fill = function(value) {

        // Steps 1-2.
        if (this == null) {
            throw new TypeError('this is null or not defined');
        }

        var O = Object(this);

        // Steps 3-5.
        var len = O.length >>> 0;

        // Steps 6-7.
        var start = arguments[1];
        var relativeStart = start >> 0;

        // Step 8.
        var k = relativeStart < 0 ?
            Math.max(len + relativeStart, 0) :
            Math.min(relativeStart, len);

        // Steps 9-10.
        var end = arguments[2];
        var relativeEnd = end === undefined ?
            len : end >> 0;

        // Step 11.
        var final = relativeEnd < 0 ?
            Math.max(len + relativeEnd, 0) :
            Math.min(relativeEnd, len);

        // Step 12.
        while (k < final) {
            O[k] = value;
            k++;
        }

        // Step 13.
        return O;
    };
}

function addEvent(evnt, elem, func) {
    if (elem.addEventListener)  // W3C DOM
        elem.addEventListener(evnt,func,false);
    else if (elem.attachEvent) { // IE DOM
        elem.attachEvent("on"+evnt, func);
    }
    else { // No much to do
        elem[evnt] = func;
    }
}

function addClass(el, cl){
    el.className += " " + cl;
}

function removeClass(el, cl){
    var elClass = el.className;
    while(elClass.indexOf(' '+cl) != -1)
        elClass = elClass.replace(' '+cl, '');
    el.className = elClass;
}

function moveCursorToEnd(el) {
    if (typeof el.selectionStart == "number") {
        el.selectionStart = el.selectionEnd = el.value.length;
    } else if (typeof el.createTextRange != "undefined") {
        el.focus();
        var range = el.createTextRange();
        range.collapse(false);
        range.select();
    }
}

var chosen = "";
var scriptEl = document.createElement('script');
var suggest_div = document.getElementsByClassName('suggest_div');
var blur_div = document.getElementById("uiBlur");
var search = document.getElementById("SearchBar");
var searchElem = $("#SearchBar");
var suggestTypeElem = $("#suggest_type");
var suggest = document.getElementById("suggest");
var suggest_type = document.getElementById("suggest_type");

window.onload = function(){
    search.focus();
    document.getElementById("SearchBar").focus();
    addEvent("keydown", document.getElementById("SearchBar"), function(e){
        var key_code = e.keyCode || e.which;
        // alert(e.keyCode)
        if(key_code == 13) { //Enter keycode
           /* if(chosen !== ""){
                if(document.getElementsByClassName('suggest').length == 1){
                    search.value =  suggest_div.item(chosen).textContent;
                }
            }*/
            OpenSearchResult(search.value);
        }
        else if(key_code == 40 || key_code == 38){

            var selected_div;
            var result;
            suggest_type.value = "";

            suggest_div = document.getElementsByClassName('suggest_div');
            if (e.keyCode == 40) {

                if (chosen === "") {
                    chosen = 1;
                } else if (parseInt(chosen) < suggest_div.length-1) {
                    chosen++;
                }
                else if(parseInt(chosen) == suggest_div.length-1){
                    chosen = 0;
                }

                for (var i = 1; i < suggest_div.length; i++) {
                    removeClass(suggest_div[i], "selected");
                }

                selected_div = suggest_div[chosen];
                addClass(selected_div, "selected");
                result = selected_div.getAttribute("data-val");
                search.value = result;

                return false;

            }
            else if(e.keyCode == 38){

                e.preventDefault ? e.preventDefault() : (e.returnValue = false);

                if (chosen === "") {
                    chosen = 1;
                } else if (chosen > 0) {
                    chosen--;
                }
                else if (chosen==0){
                    chosen = suggest_div.length-1;
                }

                for (var j = 1; j < suggest_div.length; j++) {
                    removeClass(suggest_div[j], "selected");
                }
                selected_div =  suggest_div[chosen];
                addClass(selected_div, "selected");
                result = selected_div.getAttribute("data-val");
                search.value = result;

                return false;
            }
        }


    });

    

    window.JsonCallback = function(json) {

        var scripts = document.body.getElementsByTagName('script');
        var index = scripts.length - 1;
        var search_string = search.value;

        suggest.innerHTML = "";

        var suggest_api_obj_key = Object.keys(json)[0];
        var suggest_api_obj;

        if(json[1] == undefined){
            suggest_api_obj = json[suggest_api_obj_key]["results"]
        }
        else{
            suggest_api_obj = json[1];
        }

        suggest_api_obj.forEach(logArrayElements);

        if(suggest_api_obj.length > 0){
            showSuggest()
            if(suggest_api_obj[0]["key"].indexOf(search_string) == 0){
                suggest_type.value = suggest_api_obj[0]["key"];
            }
            else
                suggest_type.value = "";
        }
        else{
            hideSuggest();
        }

        function logArrayElements(value, key, array) {

            value = value["key"];

            var input_str = search.value;
            var input_words = input_str.replace(/\s\s+/g, ' ').trim().split(" ");
            var suggest_words = value.split(" ");

            var display_value = "";

            var length = suggest_words.length;
            var count_array = Array(length+1).fill(0);
            var count_input_array = Array(input_words.length+1).fill(0);
            var match_length_array = Array(input_words.length+1).fill(0);

            var i1, i2, len1, len2;

            for (i2 = 0, len2 = input_words.length; i2 < len2; i2++) {

                for (i1 = 0, len1 = suggest_words.length; i1 < len1; i1++) {
                    if(count_array[i1] == 0 &&  suggest_words[i1] == input_words[i2]){
                        count_array[i1] = 1;
                        count_input_array[i2] = 1;
                        break;
                    }
                }
            }

            var suggest_words_new = suggest_words.slice(0);

            for (var o = 0, len3 = count_input_array.length; o < len3; o++) {
                if(count_input_array[o] == 0){

                    var word_input = input_words[o];

                    var pervious = -1;
                    var test = "";
                    var index = -1;


                    for ( i2 = 0, len2 = suggest_words.length; i2 < len2; i2++) {

                        var suggest_input = suggest_words[i2];
                        var string = "";

                        var pos = suggest_input.indexOf(word_input);

                        if(pos >= 0 && word_input.length > match_length_array[i2]){

                            index = i2;
                            match_length_array[i2] = word_input.length;


                            if(pos == 0){

                                test = escapeHTML(word_input) + "<b>"+  escapeHTML(suggest_input.substring(word_input.length, suggest_input.length)) +"</b>";

                            }
                            else if(pos + word_input.length == suggest_input.length){

                                test = "<b>"+  escapeHTML(suggest_input.substring(0, suggest_input.length-word_input.length)) +"</b>" + escapeHTML(word_input);
                            }
                            else{

                                test =  "<b>" + escapeHTML(suggest_input.substring(0, suggest_input.length-word_input.length-1)) + "</b>" + escapeHTML(word_input) + "<b>" +  escapeHTML(suggest_input.substring(pos+word_input.length)) +"</b>";

                            }
                        }
                        pervious = string.length;
                    }

                    count_array[index] = 1;
                    suggest_words_new[index] = test;
                }
            }

            for (var i = 0, len = suggest_words.length; i < len; i++) {
                if(count_array[i] == 0){
                    display_value += " <b>" + (suggest_words_new[i]) +"</b>";
                }
                else
                    display_value += " " + (suggest_words_new[i]);
            }

            var div = document.createElement("div");
            div.className = "suggest_div";
            div.setAttribute('data-index', key+1);
            div.setAttribute('data-val', value);
            div.onclick = function() { OpenSearchResult(value)};
            div.innerHTML = display_value;

            suggest.appendChild(div);

            addEvent("mouseover", div, function(e){

                suggest_div = document.getElementsByClassName('suggest_div');

                for (var k = 0; k < suggest_div.length; k++) {
                    removeClass(suggest_div[k], "selected");
                }

                var targ;

                if (!e)
                    e = window.event;
                if (e.target)
                    targ = e.target;
                else if (e.srcElement)
                    targ = e.srcElement;

                if (targ.nodeType == 3) // defeat Safari bug
                    targ = targ.parentNode;

                addClass(div, "selected");
                search.value = div.textContent
                document.getElementById("old_value").value = "";
                chosen = targ.getAttribute("data-index");
            });
        }
    };

    var resultsSelected = false;

    addEvent("mouseover", suggest, function(){
        resultsSelected = true;

    });

    addEvent("mouseleave", suggest, function(){

        var elem = document.getElementsByClassName('selected');
        if(elem.length == 1){
            removeClass(elem[0], "selected");
        }
        resultsSelected = false;
    });


    addEvent("focus", suggest_type, function(){
        search.focus();
        moveCursorToEnd(search);
    });

    search.onblur = function(){
        if (!resultsSelected) {  //if you click on anything other than the results
            hideSuggest();
        }
    };

    search.onfocus = function() {
        if(this.value != ""){
            showSuggest()
        }
    };

    document.getElementById('img-right-button').addEventListener("click", function(){
        if (search.value !== "") {
            OpenSearchResult(search.value);
        }

    });


    var cssBlur = new CssGenerator("blur", true);
    cssBlur.init()

    window.addEventListener('resize', function() {
        generated = false;
        clipToElement("#uiBlur", document.querySelector("#suggest"));
    });

};

function OpenSearchResult(searchQuery) {
    startSearch(searchQuery);
}

function getUrlVars(url) {
    var vars = {};
    url.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m,key,value) {
        vars[key] = value;
    });
    return vars;
}

function escapeHTML(str) { return str.replace(/[&"'<>]/g, (m) => ({ "&": "&amp;", '"': "&quot;", "'": "&#39;", "<": "&lt;", ">": "&gt;" })[m]); }

var css = {}

function resizeBlur() {
    clipToElement("#uiBlur", document.querySelector("#suggest"));
}

function clipToElement(selector, element){
    var params = [element.offsetTop, element.offsetLeft+element.offsetWidth, element.offsetTop+element.offsetHeight, element.offsetLeft];
    var clip = { clip: "rect("+params.join("px,") + 'px' +")" };
    addSelector(selector, clip);
}
var generated = false;
function addSelector(selector, keys){
    css[selector] = {
        keys: keys
    };
    if(!generated){
        generateCss();
    }

}

function generateCss() {
    var result = "";
    for (var selector in css) {
        if (css.hasOwnProperty(selector)) {
            result += selector + "{";
            for (var key in css[selector].keys) {
                if (css[selector].keys.hasOwnProperty(key)) {
                    result += key + ': ' + css[selector].keys[key] + ';';
                }
            }
            result += "}";
        }
    }
    document.getElementById('blur').innerHTML = result;
    generated = true;
}

function CssGenerator(id, autoGenerate, unittests){ //jshint ignore: line
    var styleElement = document.getElementById(id);
    var css = {};

    function init(){
        if(!styleElement){
            styleElement = document.createElement("style");
            styleElement.setAttribute("id", id);

            document.querySelector("head").appendChild(styleElement);
        }
    }

    function addSelector(selector, keys){
        css[selector] = {
            keys: keys
        };
        if(autoGenerate){
            generateCss();
        }
    }

    function editSelectorKeyValue(selector, key, value){
        css[selector].keys[key] = value;
        if(autoGenerate){
            generateCss();
        }
    }

    function removeSelector(selector){
        delete css[selector];
        if(autoGenerate){
            generateCss();
        }
    }

    function generateCss(){
        var result ="";
        for(var selector in css){
            if (css.hasOwnProperty(selector)) {
                result += selector+"{";
                for(var key in css[selector].keys){
                    if (css[selector].keys.hasOwnProperty(key)) {
                        result += key+': '+css[selector].keys[key]+';';
                    }
                }
                result += "}";
            }
        }
        styleElement.innerHTML = result;
    }

    this.addSelector = addSelector;
    this.editSelectorKeyValue = editSelectorKeyValue;
    this.removeSelector = removeSelector;
    this.generateCss = generateCss;
    this.init = init;
    if(unittests){
        this.css = css;
        this.styleElement = styleElement;
        this.syncData = function(){
            css = this.css;
            styleElement = this.styleElement;
        };
    }
    return this;
}

function showSuggest(){
    searchElem.css("border-bottom-left-radius", "0px");
    searchElem.css("border-bottom-right-radius", "0px");
    suggestTypeElem.css("border-bottom-left-radius", "0px");
    suggestTypeElem.css("border-bottom-right-radius", "0px");
    suggest.style.display = "block";
    blur_div.style.display = "block";
    resizeBlur()
}

function hideSuggest(){
    searchElem.css("border-bottom-left-radius", "15px");
    searchElem.css("border-bottom-right-radius", "15px");
    suggestTypeElem.css("border-bottom-left-radius", "15px");
    suggestTypeElem.css("border-bottom-right-radius", "15px");
    suggest.style.display = "none";
    blur_div.style.display = "none";
}





